<?php
  /**
   * This class is a wrapper around PHP's libCURL library.
   * It provides helper functions such as server to server upload, POST forwarding etc.
   *
   * @author Macdonald Robinson {@link http://macs-framework.sourceforge.net/}
  */
  class CURL
  {
    private $errMessages = array();

    private $host;
    private $username;
    private $password;
    private $protocol;
    private $portNumber;
    private $basePath;

    public function __construct($host, $username, $password, $protocol='ftp', $portNumber='21', $basePath='/')
    {
      $this->host = $host;
      $this->username = $username;
      $this->password = $password;
      $this->protocol = $protocol;
      $this->portNumber = $portNumber;
      $this->basePath = $basePath;
    }

    public function getURL()
    {
      $url="$this->protocol://$this->username:$this->password@$this->host:$this->portNumber$this->basePath";
      return $url;
    }

    public function getHost()
    {
      return $this->host;
    }

    public function getUsername()
    {
      return $this->username;
    }

    public function getPassword()
    {
      return $this->password;
    }

    public function getProtocol()
    {
      return $this->protocol;
    }

    public function getPortNumber()
    {
      return $this->portNumber;
    }

    public function getBasePath()
    {
      return $this->basePath;
    }
    
    /**
     * @return array
    */
    public function getErrorMessages()
    {
      return $this->errMessages;
    }

    private function setErrorMessage($ch)
    {
      $this->setCustomErrorMessage(curl_error($ch), curl_errno($ch));
    }

    private function setCustomErrorMessage($errorMessage, $errorNumber='-1')
    {
      $error = array();
      $error['Number'] = $errorNumber;
      $error['Message'] = $errorMessage;
      
      $this->errMessages[] = $error;
    }

    /**
     * @return string
    */
    public static function getEncoded($oned_array)
    {
      $str = '';
      foreach($oned_array as $key=>$val)
      {
        $str .= urlencode($key).'='.urlencode($val).'&';
      }
      return $str;
    }
    
    /**
     * The second argument is a One Dimentional array of key value pairs
     *
     * @return bool|mixed
    */
    public static function httpPostRequest($url, $data)
    {
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_POST, 1);
      curl_setopt($ch, CURLOPT_POSTFIELDS, self::getEncoded($data));
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      $result = curl_exec($ch);

      curl_close($ch);
      return $result;
    }
    
    /**
     * @return mixed|bool
    */
    public static function httpRequest($url)
    {
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 0);
      $result = curl_exec($ch);

      curl_close($ch);
      return $result;
    }
    
    /**
     * @return mixed|bool
    */
    public function uploadFile($local_path, $target_path, $additionalCurlOptions = array())
    {
      if(file_exists($local_path))
        $fp = fopen($local_path, "r");
      else
      {
        $this->setCustomErrorMessage("file '$local_path' does not exist");
        return false;
      }
      
      $url= $this->getURL().$target_path;
      $ch = curl_init();
      
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);      
      curl_setopt($ch, CURLOPT_USERPWD, $this->username.':'.$this->password);
      curl_setopt($ch, CURLOPT_UPLOAD, 1);
      curl_setopt($ch, CURLOPT_INFILE, $fp);
      curl_setopt($ch, CURLOPT_TRANSFERTEXT, 1);
      curl_setopt($ch, CURLOPT_INFILESIZE, filesize($local_path));

      curl_setopt_array($ch, $additionalCurlOptions);

      $result = curl_exec($ch);

      if((!$result) && (curl_errno($ch) !== 0 ))
        $this->setErrorMessage($ch);

      curl_close($ch);
      return $result;
    }

    /**
     * @return mixed|bool
    */
    public function recursiveUpload($local_path, $target_path, $additionalCurlOptions = array())
    {
      if (( !file_exists($local_path) ) || ( !is_dir($local_path) ))
        return false;

      $allPaths = $this->scanDirectories($local_path);

      foreach($allPaths['dirs'] as $path)
      {
        $localPath = $path;
        $targetPath = str_replace($local_path, $target_path, $path);

        $return = $this->makeFtpDirectory($targetPath);

        if(!$return)
          return false;
      }
      
      foreach($allPaths['files'] as $path)
      {
        $localPath = $path;
        $targetPath = str_replace($local_path, $target_path, $path);

        $this->uploadFile($localPath, $targetPath, $additionalCurlOptions);
      }
      
      return true;
    }

    public function makeFtpDirectory($dirPath)
    {
      $url= $this->getURL();

      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      $cmd = array();
      $cmd[] = 'MKD '.$dirPath;
      curl_setopt($ch, CURLOPT_POSTQUOTE, $cmd);

      $result = curl_exec($ch);      

      if((!$result) && (curl_errno($ch) !== 0 ))
        $this->setErrorMessage($ch);
      
      curl_close($ch);
      return $result;
    }

    private function scanDirectories($rootDir, $allData=array()) {
        // set filenames invisible if you want
        $invisibleFileNames = array(".", "..", ".htaccess", ".htpasswd");
        // run through content of root directory
        $dirContent = scandir($rootDir);
        $allData['dirs'][] = $rootDir;
        foreach($dirContent as $key => $content) {
            // filter all files not accessible
            $path = $rootDir.'/'.$content;
            if(!in_array($content, $invisibleFileNames)) {
                // if content is file & readable, add to array
                if(is_file($path) && is_readable($path)) {
                    // save file name with path
                    $allData['files'][] = $path;
                // if content is a directory and readable, add path and name
                }elseif(is_dir($path) && is_readable($path)) {
                    // recursive callback to open new directory                    
                    $allData = $this->scanDirectories($path, $allData);
                }
            }
        }
        return $allData;
    }
  }
?>
